﻿using MC_Launcher_v2.Business.Minecraft.Net;
using MC_Launcher_v2.Business.Minecraft.Profiles;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.NetworkInformation;
using System.Text;
using System.Threading;

namespace MC_Launcher_v2.Business.Minecraft
{
    public class MinecraftManager
    {
        #region METHODS
        public Authentication Authenticate(string email, string password)
        {
            JObject obj = new JObject();
            JObject agent = new JObject();
            agent.Add("name", "Minecraft");
            agent.Add("version", 1);
            obj.Add("agent", agent);
            obj.Add("username", email);
            obj.Add("password", password);
            obj.Add("requestUser", true);
            return Request.Post<Authentication>("https://authserver.mojang.com/authenticate", obj);
        }
        public ErrorMessage RetrieveError(WebException e)
        {
            using (WebResponse r = e.Response)
            {
                HttpWebResponse resp = (HttpWebResponse)r;
                string content = "";
                using (var str = resp.GetResponseStream())
                using (var reader = new StreamReader(str))
                    content = reader.ReadToEnd();
                return (ErrorMessage)JsonConvert.DeserializeObject(content, typeof(ErrorMessage));
            }
        }
        public bool AuthenticationValid(string session)
        {
            JObject obj = new JObject();
            obj.Add("accessToken", session);
            return Request.SimplePost("https://authserver.mojang.com/validate", obj);
        }
        public bool Signout(string username, string password)
        {
            JObject obj = new JObject();
            obj.Add("username", username);
            obj.Add("password", password);
            return Request.SimplePost("https://authserver.mojang.com/signout", obj);
        }
        public ProcessStartInfo BuildStartInfo(string gameDir, JavaProfile java, Profiles.UserProfile user)
        {

            StringBuilder builder = new StringBuilder();
            builder.Append(java.Args.Trim());
            builder.AppendFormat(" -Xmx{0}M -Xms{0}M", java.Ram);
            //TODO: -djava.library.path
            builder.Append(" -cp ");
            var files = Directory.GetFiles(gameDir + "\\libraries", "*.jar", SearchOption.AllDirectories).Concat(new string[] { gameDir + "\\versions\\" + java.LibVersion });
            builder.AppendFormat("\"{0}\"", string.Join(";", files));
            builder.Append(" net.minecraft.launchwrapper.Launch");
            builder.AppendFormat(" --username {0}", user.DisplayName);
            builder.AppendFormat(" --version {0}", java.Version);
            builder.AppendFormat(" --gameDir \"{0}\"", gameDir);
            builder.AppendFormat(" --assetsDir \"{0}\"", gameDir + "\\assets");
            var file = new FileInfo(gameDir + "\\versions\\" + java.LibVersion);
            builder.AppendFormat(" --assetIndex {0}", file.Name.Substring(0, file.Name.Length - file.Extension.Length));
            builder.AppendFormat(" --uuid {0}", user.UUID);
            builder.AppendFormat(" --accessToken {0}", user.AccessToken);
            builder.Append(" --userProperties {}");
            builder.Append(" --userType mojang");

            ProcessStartInfo info = new ProcessStartInfo();
            info.FileName = java.Executable;
            info.Arguments = builder.ToString();
            info.UseShellExecute = false;
            info.WorkingDirectory = gameDir;

            File.WriteAllText("args.txt", info.Arguments);

            return info;
        }
        #endregion
    }
}
