﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using MetroFramework.Controls;
using MC_Launcher_v2.Business.Minecraft.Profiles;
using MC_Launcher_v2.Business.Minecraft;
using System.Threading;
using System.Net;

namespace MC_Launcher_v2.Presentation
{
    public partial class pnlLogin : MetroUserControl
    {
        #region VARIBALES
        private List<UserProfile> profiles;
        #endregion

        #region PROPERTIES
        public List<UserProfile> UserProfiles
        {
            get { return profiles; }
            set
            {
                if (profiles != value)
                {
                    profiles = value;

                    cbbProfiles.Items.Clear();
                    cbbProfiles.Items.AddRange(profiles.ToArray());
                    if (profiles.Count > 0)
                        cbbProfiles.SelectedItem = profiles[0];
                }
            }
        }
        public UserProfile SelectedProfile { get; set; }
        public EventHandler SuccessfullLogin { get; set; }
        #endregion

        public pnlLogin()
        {
            InitializeComponent();
        }

        public void ShowError(string format, params object[] args)
        {
            lblErrorLeft.Visible = true;
            lblErrorRight.Visible = true;
            lblErrorRight.Text = string.Format(format, args);
        }

        public void HideError()
        {
            lblErrorLeft.Visible = lblErrorRight.Visible = false;
            lblErrorRight.Text = "";
        }

        private void cbbProfiles_SelectedIndexChanged(object sender, EventArgs e)
        {
            UserProfile profile = cbbProfiles.SelectedItem as UserProfile;
            if (profile == null)
            {
                cbbProfiles.Items.RemoveAt(cbbProfiles.SelectedIndex);
                ShowError("Fehlerhafte Form-Daten");
            }
            else
            {
                txbUsername.Text = profile.Username;
                cbbProfiles.Enabled = false;
                btnChose.Enabled = false;
                btnChose.Style = MetroFramework.MetroColorStyle.Orange;
                Wait(() =>
                {
                    MinecraftManager m = new MinecraftManager();
                    bool isValid = false;
                    try { isValid = m.AuthenticationValid(profile.AccessToken); } catch { }
                    this.InvokeSafe(() =>
                    {
                        if (!isValid)
                        {
                            ShowError("Account \"{0}\" muss neu eingeloggt werden!", profile.DisplayName);
                            cbbProfiles.Items.RemoveAt(cbbProfiles.SelectedIndex);
                            txbUsername.Text = profile.Username;
                        }
                        else
                        {
                            //Works
                            SelectedProfile = profile;
                            btnChose.Style = MetroFramework.MetroColorStyle.Green;
                            btnChose.Enabled = true;
                        }
                        cbbProfiles.Enabled = true;
                    });
                });
            }
        }

        void Wait(Action action)
        {
            Thread t = new Thread(() => {
                this.InvokeSafe(() => lblSpinner.Visible = true);
                action();
                this.InvokeSafe(() => lblSpinner.Visible = false);
            });
            t.Name = "[UI] Background";
            t.IsBackground = true;
            t.Start();
        }

        private void btnLogin_Click(object sender, EventArgs e)
        {
            string mail = txbUsername.Text.Trim();
            string pass = txbPassword.Text.Trim();
            HideError();
            if (string.IsNullOrEmpty(mail))
            {
                ShowError("Gib einen Usernamen an!");
                return;
            }
            if (string.IsNullOrEmpty(pass))
            {
                ShowError("Gib ein Passwort an!");
                return;
            }

            btnLogin.Enabled = false;
            Wait(() =>
            {
                MinecraftManager m = new MinecraftManager();
                try
                {
                    var auth = m.Authenticate(mail, pass);
                    UserProfile p = new UserProfile();
                    p.AccessToken = auth.accessToken;
                    p.ClientToken = auth.clientToken;
                    p.DisplayName = auth.selectedProfile.name;
                    p.UserID = auth.user.id;
                    p.UUID = auth.selectedProfile.id;
                    p.Username = mail;
                    SelectedProfile = p;
                    this.InvokeSafe(() => 
                    {
                        cbbProfiles.Items.Add(p);
                        cbbProfiles.SelectedIndex = cbbProfiles.Items.IndexOf(p);
                        SuccessfullLogin?.Invoke(this, new EventArgs());
                    });
                }
                catch(WebException wex)
                {
                    var err = m.RetrieveError(wex);
                    this.InvokeSafe(() => ShowError("Ein Fehler ist aufgetreten:\n{0}\n{1}", err.error, err.errorMessage));
                }
                
                this.InvokeSafe(() => btnLogin.Enabled = true);
            });

        }

        private void btnChose_Click(object sender, EventArgs e)
        {
            btnChose.Style = MetroFramework.MetroColorStyle.Orange;
            UserProfile profile = cbbProfiles.SelectedItem as UserProfile;
            if (profile == null)
            {
                cbbProfiles.Items.RemoveAt(cbbProfiles.SelectedIndex);
                ShowError("Fehlerhaftes Profil");
            } else
            {
                SelectedProfile = profile;
                btnChose.Style = MetroFramework.MetroColorStyle.Green;
                SuccessfullLogin?.Invoke(this, new EventArgs());
            }
        }
    }
}
